@@ -709,8 +709,8 @@ key_state_serialise (KeyState *state)
               gsize size;
               gsize i;
 
-              data = state->strinfo->str;
               size = state->strinfo->len;
+              data = g_string_free_and_steal (g_steal_pointer (&state->strinfo));
 
               words = data;
               for (i = 0; i < size / sizeof (guint32); i++)
@@ -720,9 +720,6 @@ key_state_serialise (KeyState *state)
                                                data, size, TRUE,
                                                g_free, data);
 
-              g_string_free (state->strinfo, FALSE);
-              state->strinfo = NULL;
-
               g_variant_builder_add (&builder, "(y@au)",
                                      state->is_flags ? 'f' :
                                      state->is_enum ? 'e' : 'c',
@@ -1870,7 +1867,7 @@ compare_strings (gconstpointer a,
                  gconstpointer b)
 {
   const gchar *one = a;
-  const gchar *two = a;
+  const gchar *two = b;
   gint cmp;
 
   cmp = g_str_has_suffix (two, ".enums.xml") -
