@@ -1469,7 +1469,7 @@ typedef struct
   GCancellable *enumeration_parent_cancellable;  /* (nullable) (owned) */
   gulong enumeration_cancelled_id;
 
-  GSList *connection_attempts;
+  GSList *connection_attempts;  /* (element-type ConnectionAttempt) (owned) */
   GSList *successful_connections;
   SocketClientErrorInfo *error_info;
 
@@ -1510,6 +1510,7 @@ typedef struct
   GSocketClientAsyncConnectData *data; /* unowned */
   GSource *timeout_source;
   GCancellable *cancellable;
+  GCancellable *task_cancellable;  /* (owned); this is equal to g_task_get_cancellable (ConnectionAttempt.data->task), but with a longer lifetime */
   gulong cancelled_id;
   grefcount ref;
 } ConnectionAttempt;
@@ -1538,7 +1539,8 @@ connection_attempt_unref (gpointer pointer)
       g_clear_object (&attempt->address);
       g_clear_object (&attempt->socket);
       g_clear_object (&attempt->connection);
-      g_cancellable_disconnect (g_task_get_cancellable (attempt->data->task), attempt->cancelled_id);
+      g_cancellable_disconnect (attempt->task_cancellable, attempt->cancelled_id);
+      g_clear_object (&attempt->task_cancellable);
       attempt->cancelled_id = 0;
       g_clear_object (&attempt->cancellable);
       g_clear_object (&attempt->proxy_addr);
@@ -1872,7 +1874,7 @@ g_socket_client_connected_callback (GObject      *source,
 				    GAsyncResult *result,
 				    gpointer      user_data)
 {
-  ConnectionAttempt *attempt = user_data;
+  ConnectionAttempt *attempt = g_steal_pointer (&user_data);
   GSocketClientAsyncConnectData *data = attempt->data;
 
   if (task_completed_or_cancelled (data) || g_cancellable_is_cancelled (attempt->cancellable))
@@ -2030,12 +2032,15 @@ g_socket_client_enumerator_callback (GObject      *object,
 
   g_source_set_callback (attempt->timeout_source, on_connection_attempt_timeout, attempt, NULL);
   g_source_attach (attempt->timeout_source, g_task_get_context (data->task));
-  data->connection_attempts = g_slist_append (data->connection_attempts, attempt);
+  data->connection_attempts = g_slist_append (data->connection_attempts, connection_attempt_ref (attempt));
 
   if (g_task_get_cancellable (data->task))
-    attempt->cancelled_id =
-        g_cancellable_connect (g_task_get_cancellable (data->task), G_CALLBACK (on_connection_cancelled),
-                               g_object_ref (attempt->cancellable), g_object_unref);
+    {
+      attempt->task_cancellable = g_object_ref (g_task_get_cancellable (data->task));
+      attempt->cancelled_id =
+          g_cancellable_connect (attempt->task_cancellable, G_CALLBACK (on_connection_cancelled),
+                                 g_object_ref (attempt->cancellable), g_object_unref);
+    }
 
   g_socket_connection_set_cached_remote_address ((GSocketConnection *)attempt->connection, address);
   g_debug ("GSocketClient: Starting TCP connection attempt");
@@ -2043,7 +2048,7 @@ g_socket_client_enumerator_callback (GObject      *object,
   g_socket_connection_connect_async (G_SOCKET_CONNECTION (attempt->connection),
 				     address,
 				     attempt->cancellable,
-				     g_socket_client_connected_callback, connection_attempt_ref (attempt));
+				     g_socket_client_connected_callback, attempt  /* transfer full */);
 }
 
 /**
