@@ -41,6 +41,7 @@ struct  _GRealThread
 /* system thread implementation (gthread-posix.c, gthread-win32.c) */
 
 #if defined(HAVE_FUTEX) || defined(HAVE_FUTEX_TIME64)
+#include <errno.h>
 #include <linux/futex.h>
 #include <sys/syscall.h>
 #include <unistd.h>
@@ -64,9 +65,13 @@ struct  _GRealThread
 #define g_futex_simple(uaddr, futex_op, ...)                                     \
   G_STMT_START                                                                   \
   {                                                                              \
+    int saved_errno = errno;                                                     \
     int res = syscall (__NR_futex_time64, uaddr, (gsize) futex_op, __VA_ARGS__); \
     if (res < 0 && errno == ENOSYS)                                              \
-      syscall (__NR_futex, uaddr, (gsize) futex_op, __VA_ARGS__);                \
+      {                                                                          \
+        errno = saved_errno;                                                     \
+        syscall (__NR_futex, uaddr, (gsize) futex_op, __VA_ARGS__);              \
+      }                                                                          \
   }                                                                              \
   G_STMT_END
 #elif defined(__NR_futex_time64)
@@ -89,25 +94,10 @@ struct  _GRealThread
 
 #endif
 
-/* Platform-specific scheduler settings for a thread */
-typedef struct
-{
-#if defined(HAVE_SYS_SCHED_GETATTR)
-  /* This is for modern Linux */
-  struct sched_attr *attr;
-#elif defined(G_OS_WIN32)
-  gint thread_prio;
-#else
-  /* TODO: Add support for macOS and the BSDs */
-  void *dummy;
-#endif
-} GThreadSchedulerSettings;
-
 void            g_system_thread_wait            (GRealThread  *thread);
 
 GRealThread *g_system_thread_new (GThreadFunc proxy,
                                   gulong stack_size,
-                                  const GThreadSchedulerSettings *scheduler_settings,
                                   const char *name,
                                   GThreadFunc func,
                                   gpointer data,
@@ -117,19 +107,14 @@ void            g_system_thread_free            (GRealThread  *thread);
 void            g_system_thread_exit            (void);
 void            g_system_thread_set_name        (const gchar  *name);
 
-gboolean        g_system_thread_get_scheduler_settings (GThreadSchedulerSettings *scheduler_settings);
-
 /* gthread.c */
 GThread *g_thread_new_internal (const gchar *name,
                                 GThreadFunc proxy,
                                 GThreadFunc func,
                                 gpointer data,
                                 gsize stack_size,
-                                const GThreadSchedulerSettings *scheduler_settings,
                                 GError **error);
 
-gboolean g_thread_get_scheduler_settings (GThreadSchedulerSettings *scheduler_settings);
-
 gpointer        g_thread_proxy                  (gpointer      thread);
 
 guint           g_thread_n_created              (void);
